#include "InConnection.h"
#include "Logging.h"
#include "QtellaSub.h"
#include "Gnutella.h"
#include "Address.h"
#include "UploadManager.h"
#include "DownloadManager.h"
#include "Connections.h"

#include <qtimer.h>
#include <qsocket.h>

InConnection::InConnection(int socket) : _state(Connected)
{
  _socket = new QSocket();
  _socket->setSocket(socket);
  connect(_socket, SIGNAL(readyRead()), SLOT(slotReadyRead()));

  _timeout = new QTimer(this);
  connect(_timeout, SIGNAL(timeout()), SLOT(slotTimeout()));
  _timeout->start(6000);
}  


InConnection::~InConnection()
{
  if( _socket ) delete _socket;
  if( _timeout ) delete _timeout;
}


void InConnection::slotReadyRead()
{
  QtellaSub* p = QtellaSub::getInstance();


  int     r = 0;
  char    buffer[1024];

  r = _socket->readBlock(buffer, 1024);
  _rec.append(buffer, r);

  // incoming connection
  if(_rec.find(CONNECT_STR) != std::string::npos)
    {
      _timeout->stop();
      disconnect(_socket, 0, 0, 0);
      Logging::getInstance()->log(Logging::Debug, "New incoming connection.");
      std::string ip(_socket->peerAddress().toString());
      p->connections->addServant(Address(ip, _socket->port()), _socket);

      _socket = NULL;
      _state = Closed;
      return;
    }

  // GET
  if( _rec.find("\r\n\r\n") != std::string::npos )
    {
      disconnect(_socket, 0, 0, 0);
      
      if(_rec.substr(0, 3) == "GET")
	{
	  _timeout->stop();
	  Logging::getInstance()->log(Logging::Debug, "New upload request.");
	  p->upload_manager->addUpload(_rec, _socket);

	  _socket = NULL;
	  _state = Closed;
	  return;
	}
    }

  if( _rec.find("\n\n") != std::string::npos )
    {
      disconnect(_socket, 0, 0, 0);

      if(_rec.substr(0, 3) == "GIV")
	{
	  _timeout->stop();
	  Logging::getInstance()->log(Logging::Debug, "Got GIV request.");
	  p->download_manager->updatePush(_rec, _socket);

	  _socket = NULL;
	  _state = Closed;
	  return;
	}
    }
}


void InConnection::slotTimeout()
{
  _timeout->stop();
  Logging::getInstance()->log(Logging::Error, "Incoming connection timed out!");

  if(_socket)
    {
      delete _socket;
      _socket = NULL;
    }

  _state = Closed;
}


int InConnection::state()
{
  return _state;
}
